import getUrlContents from '../cdn/getUrlContents';
import { Product } from '../interfaces/ISettings';
import { SsnDiffType } from '../interfaces/ISsnFileEntry';
import extractFile from './extractFile';
import getSolidpkg from './getSolidpkg';
import readSsnFile from './reader/readSsnFile';

export default async function getPatch(product: Product, from: number, to: number) {
  const solidpkg = await getSolidpkg(product, from, to);
  console.debug(solidpkg.files);

  const downloadPromises = solidpkg.files.map((file) => getUrlContents({ host: 'cdn-patch.swtor.com', path: `/patch/${product}/${product}_${from}to${to}/${file.name}` }));
  const bufferArray = await Promise.all(downloadPromises);
  const zipFile = bufferArray[bufferArray.length - 1];
  const dvArray = bufferArray.map((buffer) => new DataView(buffer));

  //TODO: Verify that downloaded files match the hash in `solidpkg.pieces`

  const fileEntries = readSsnFile(zipFile);
  console.debug(fileEntries);

  //Verify file entries
  if (from === -1) {
    fileEntries.filter((file) => file.diffType !== SsnDiffType.NewFile).forEach((file) => {
      throw new Error(`Patches from -1 must be included in full, but this patch had a file "${file.name}" with diff type ${file.diffType}.`);
    });
  }
  //TODO: last file must always be `${product}.version` with diff type. Other files depend on diffType.

  //Extract newly added files
  fileEntries.filter((file) => file.diffType === SsnDiffType.NewFile).forEach(async (file) => {
    const fileContents = await extractFile(file, dvArray);
    console.debug(new Uint8Array(fileContents));
    //TODO
  });

  //Extract changed files
  fileEntries.filter((file) => file.diffType === SsnDiffType.Changed).forEach(async (file) => {
    const fileContents = await extractFile(file, dvArray);
    console.debug(new Uint8Array(fileContents));
    //TODO
  });

  //Need to delete deleted files
  fileEntries.filter((file) => file.diffType === SsnDiffType.Deleted).forEach((file) => {
    //TODO
  });
}
